/*
   	Tairo v0.61
    Copyright (C) 2004 Jonas Gehring
        
    Tairo is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Tairo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tairo; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 30.10.04; 14:56:45

// Own headers
#include "tairo.h"						// Constants, proto-types etc.
#include "gfx.h"							// Sprite definitions
#include "levels.h"						// The data for the levels
// Biult-in headers
#include <graph.h>						// Common (non-windowed) graphic routines
#include <gray.h>							// Routines for grayscale graphics
#include <kbd.h>							// Keyboard handling routines
#include <mem.h>							// Routines for manipulation of memory blocks
#include <peekpoke.h>					// Routines for easier access to the memory and to I/O ports
#include <system.h>						// System routines (for accessing system timers, queues, etc.)
#include <stdio.h>						// ANSI-compatible file and TTY input/output routines
// Other headers
#include "extgraph.h"					// Include ExtGraph v2.00beta4 by TI-Chess Team


// Globals for the cursor position and for the undo-function
short cursx = 2, cursy = 0;
short clevel[11][11];
short tempop[4];	// X, Y, Direction, Undo (True or False)


// "Main routine" for the game
void GamePlay(START_OPTIONS options)
{
	short x = 2, y = 0;
	char coinnr[3];
	
	timeout = FALSE;
	memcpy(clevel, levels[options.level], 11*11*(sizeof(short)));
	
	tempop[3] = FALSE;

	DrawField(options.level);
	
	// Timer for the Cursor
	OSVRegisterTimer(1, 7, CursorBlink);
	
	if (options.mode == 2) {
		GrayDrawStrExt(120+CALCX, 56+CALCY, "10", A_SHADOWED | A_REPLACE, F_6x8);
		OSVRegisterTimer(2, 24-(options.diff*3), StopTimer);
	}
	
	// Find starting position
	do {
		x++;
		
		if (clevel[y+2][x+2] == UNVALID) {
			x = 0;
			y++;
		}
		if (y > 7)
			return;
		
	} while (clevel[y+2][x+2] != FILLED);
	
	cursy = y;
	cursx = x;
		
	pieces_left = 0;
	// Draw the stones onto the field and calculate the coins that exist
	for (y = 0; y < 7; y++) {
		for (x = 0; x < 7; x++)
			if (clevel[y+2][x+2] == FILLED)
				pieces_left++;
	}
	
	if (pieces_left > 9)
		sprintf(coinnr, "%d ", pieces_left);
	else
		sprintf(coinnr, " %d ", pieces_left);
	GrayDrawStrExt(120+CALCX, 24+CALCY, coinnr, A_SHADOWED | A_REPLACE, F_6x8);


	// Game loop
	do {		

		for (y = 0; y < 7; y++)
			for (x = 0; x < 7; x++)
			if (clevel[y+2][x+2] != UNVALID) {
				GraySprite8_RPLC_R(19+CALCX+(x*9), 19+CALCY+(y*9), HEIGHT, GetSpr(LIGHT_PLANE, clevel[y+2][x+2]), GetSpr(DARK_PLANE, clevel[y+2][x+2]), GRAYPL);
		}
		
		// The user is unable to keep on playing
		if (CheckGameOver()) {
			break;
		}

		// The user wants to quit
		if (!SelectCoin()) {
			if (timeout)
				return;
			else {
				if (ReallyQuit())	
					break;
				else
					DrawField(options.level);
			}
		}
		
		// Show actual number of pieces that are left
  	if (pieces_left > 9)
			sprintf(coinnr, "%d ", pieces_left);
		else
			sprintf(coinnr, " %d ", pieces_left);
		GrayDrawStrExt(120+CALCX, 24+CALCY, coinnr, A_SHADOWED | A_REPLACE, F_6x8);
		
	} while (pieces_left != 1); 

	if (pieces_left == 1) {
		// The user made it!!!
		// Free the timers
		OSVFreeTimer(1);
		OSVFreeTimer(2);
		DrawBox();
		// Write into the global
		savload.wininfo[options.level] = TRUE;
		Save(&savload);	
		// Show message
		GrayDrawStrExt(0, (LCD_HEIGHT/2)-7, "Congratulations!!", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);
		GrayDrawStrExt(0, (LCD_HEIGHT/2)+1, "You have won!!", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);	
		while (Key_Wait() != KEY_ENTER) continue;
	}
}


// Selects a coin
BOOL SelectCoin(void)
{
	short key;
	
	do {
		key = Key_Wait();
		
		GraySprite8_RPLC_R(19+CALCX+(cursx*9), 19+CALCY+(cursy*9), HEIGHT, GetSpr(LIGHT_PLANE, clevel[cursy+2][cursx+2]), GetSpr(DARK_PLANE, clevel[cursy+2][cursx+2]), GRAYPL);
		
		if (key == KEY_LEFT || key == KEY_RIGHT || key == KEY_UP || key == KEY_DOWN)
			MoveCur(key);
			
		else if (key == KEY_ENTER) {
			MoveCoin();
			return TRUE;
		}	
		
		else if (key == KEY_UNDO && tempop[3]) {
			Undo();
			return TRUE;
		}
	}	while (key != KEY_ESC);
	
	return FALSE;
}


// Lets the cursor blink
CALLBACK void CursorBlink(void)
{	
	GraySprite8_XOR(19+CALCX+(cursx*9), 19+CALCY+(cursy*9), HEIGHT, GetSpr(LIGHT_PLANE, 2), GetSpr(DARK_PLANE, 2), GRAYPL);
	GraySprite8_OR(19+CALCX+(cursx*9), 19+CALCY+(cursy*9), HEIGHT, GetSpr(LIGHT_PLANE, clevel[cursy+2][cursx+2]), GetSpr(DARK_PLANE, clevel[cursy+2][cursx+2]), GRAYPL);
}


// Stops the time for 'TIME'-mode
CALLBACK void StopTimer(void)
{
	char timestr[3];
	
	time--;
	
	// Display the time which is left
	if (time > 9)
		sprintf(timestr, "%d ", time);
	else
		sprintf(timestr, "0%d ", time);

	GrayDrawStrExt(120+CALCX, 56+CALCY, timestr, A_SHADOWED | A_REPLACE, F_6x8);
}


// Moves the cursor
void MoveCur(short dir)
{
	short x = cursx, y = cursy;
	
	// Move the cursor to the next position
	do {
			
			if (dir == KEY_LEFT) x--;
			else if (dir == KEY_RIGHT) x++;
			else if (dir == KEY_UP) y--;
			else if (dir == KEY_DOWN) y++; 
	
			// Take coin to the next row or column
			if (x > 6 || y > 6 || x < 0 || y < 0) {
				if (dir == KEY_LEFT)	x = 6;
				else if (dir == KEY_RIGHT)x = 0; 	
				else if (dir == KEY_UP) y = 6;
				else if (dir == KEY_DOWN) y = 0;
			}		
				
	}	while (clevel[y+2][x+2] == UNVALID);
	
	cursx = x;
	cursy = y;
}


// Jumping process
void MoveCoin(void)
{
	short dir = SelectDir();

	if (!dir)
		return;
	
	// Store temporary values
	tempop[0] = cursx;
	tempop[1] = cursy;
	tempop[2] = dir;	
	tempop[3] = TRUE;
	
	// 1=Right, 2=Up, 3=Left, 4=Down
	pieces_left--;
	AnimateJump(cursx, cursy, dir);
	clevel[cursy+2][cursx+2] = EMPTY;
	if (dir == 1) {
		clevel[cursy+2][cursx+3] = EMPTY;
		clevel[cursy+2][cursx+4] = FILLED;
		cursx = cursx+2;
	} else if (dir == 2) {
		clevel[cursy+1][cursx+2] = EMPTY;
		clevel[cursy][cursx+2] = FILLED;
		cursy = cursy-2;
	} else if (dir == 3) {
		clevel[cursy+2][cursx+1] = EMPTY;
		clevel[cursy+2][cursx] = FILLED;
		cursx = cursx-2;
	} else if (dir == 4) {
		clevel[cursy+3][cursx+2] = EMPTY;
		clevel[cursy+4][cursx+2] = FILLED;
		cursy = cursy+2;
	}
	
	time = 11;
}


// Select direction for jump
short SelectDir(void)
{
	// 4 directions: right, up, left, down
	BOOL dirs[4] = {TRUE, TRUE, TRUE, TRUE};
	short i = 0, itemp, key, loop;

	if (clevel[cursy+2][cursx+2] != FILLED)
		return FALSE;

	// Proof if a direction is valid
	if (clevel[cursy+2][cursx+3] != FILLED || clevel[cursy+2][cursx+4] != EMPTY)
		dirs[0] = FALSE;
	if (clevel[cursy+1][cursx+2] != FILLED || clevel[cursy][cursx+2] != EMPTY)
		dirs[1] = FALSE;
	if (clevel[cursy+2][cursx+1] != FILLED || clevel[cursy+2][cursx] != EMPTY)
		dirs[2] = FALSE;
	if (clevel[cursy+3][cursx+2] != FILLED || clevel[cursy+4][cursx+2] != EMPTY)	
		dirs[3] = FALSE;
		
	// If no direction is valid, return FALSE
	if (!dirs[0] && !dirs[1] && !dirs[2] && !dirs[3])
		return FALSE;
		
	// Begin with first valid direction
	while (!dirs[i])
		i++;
	
	itemp = i;
	DrawSel(cursx, cursy, i, i);
		
	do {
		endproof:
		
		if (i != itemp) {
			DrawSel(cursx, cursy, itemp, i);
			itemp = i;
		}
	
		key = Key_Wait();

		// Find the next free directuion
		if (key == KEY_UP) {
			for (loop = 0; loop < 4; loop++) {
				i++;		
				if (i > 3)
					i = 0;
					
				if (dirs[i])
					goto endproof;
			}
			i = itemp;
		}
				
		if (key == KEY_ESC) {
			DrawSel(cursx, cursy, itemp, i);
			return 0;
		}
		
	}	while (key != KEY_ENTER);
	
	DrawSel(cursx, cursy, itemp, i);
	return i+1;
}


// Checks if the player is still able to move
BOOL CheckGameOver(void)
{
	short x, y;
	char msg[17];
	
	for (y = 2; y < 9; y++) {
		for (x = 2; x < 9; x++) {
			
			if (clevel[y][x] == FILLED) {
				
				// Check right side
				if (clevel[y+1][x] == FILLED && clevel[y+2][x] == EMPTY)
					return FALSE;

				// Check left side
				if (clevel[y-1][x] == FILLED && clevel[y-2][x] == EMPTY)
					return FALSE;
				
				// Check upper side
				if (clevel[y][x-1] == FILLED && clevel[y][x-2] == EMPTY)
					return FALSE;
				
				// Check lower side
				if (clevel[y][x+1] == FILLED && clevel[y][x+2] == EMPTY)
					return FALSE;
			}
		}
	}
	
	OSVFreeTimer(1);
	OSVFreeTimer(2);
	DrawBox();
	GrayDrawStrExt(0, (LCD_HEIGHT/2)-7, "Game Over!!!", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);
	sprintf(msg, "%d coins left...", pieces_left);
	GrayDrawStrExt(0, (LCD_HEIGHT/2)+1, msg, A_CENTERED | A_SHADOWED | A_XOR, F_4x6);
	
	while (Key_Wait() != KEY_ENTER) continue;
	return TRUE;
}


// Time's Out!
void TimeOut(void)
{
	short key;
	
	// Fre the timers
	OSVFreeTimer(1);
	OSVFreeTimer(2);

	// If we would not set time to a different value than 0, the function Key_Wait we use 
	// later in this function would call this function all the time which would cause
	// a memory error
	time = 1;

	timeout = TRUE;
	
	// Show message
	DrawBox();	
	GrayDrawStrExt(0, (LCD_HEIGHT/2)-7, "Out of time!!!", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);
	GrayDrawStrExt(0, (LCD_HEIGHT/2)+1, "Play again?", A_CENTERED | A_SHADOWED | A_XOR, F_4x6);
	
	do {
		key = Key_Wait();
	
		if (key == KEY_ENTER)
			break;
		
		if (key == KEY_ESC)
			ExitTairo();
	
	} while (TRUE);
}


// Undo-Function (1-Level)
void Undo(void)
{
	// 1=Right, 2=Left, 3=Up, 4=Down
	pieces_left++;
	clevel[tempop[1]+2][tempop[0]+2] = FILLED;
	if (tempop[2] == 1) {
		clevel[tempop[1]+2][tempop[0]+3] = FILLED;
		clevel[tempop[1]+2][tempop[0]+4] = EMPTY;
		cursx = tempop[0]+2;
	} else if (tempop[2] == 2) {
		clevel[tempop[1]+1][tempop[0]+2] = FILLED;
		clevel[tempop[1]][tempop[0]+2] = EMPTY;
		cursy = tempop[1];
	} else if (tempop[2] == 3) {
		clevel[tempop[1]+2][tempop[0]+1] = FILLED;
		clevel[tempop[1]+2][tempop[0]] = EMPTY;
		cursx = tempop[0]-2;			
	} else if (tempop[2] == 4) {
		clevel[tempop[1]+3][tempop[0]+2] = FILLED;
		clevel[tempop[1]+4][tempop[0]+2] = EMPTY;
		cursy = tempop[1];		
	} 
	
	cursx = tempop[0]; cursy = tempop[1];
	tempop[3] = FALSE;	// We can't undo more than once
	time = 11;
}


// Waits for a key and returns the value
unsigned short Key_Wait()
{
	short key = 0; 

	// Low power mode
	pokeIO(0x600005, 0b10111);

	// Wait for the keypress
	do {
				
		// The time is over!
		if (!time) {
			TimeOut();
			key = KEY_ESC;
			break;
		}
		
		if (_keytest(RR_ESC)) {
			key = KEY_ESC;
			break;
		}		
		else if (_keytest(RR_ENTER) || _keytest(RR_ENTER2)) {
			key = KEY_ENTER;
			break;
		}	
			
		else if (_keytest(RR_CLEAR)) {
			while (_rowread(0)) continue;
			off();
		}	
		
		else if (_keytest(RR_BCKSPC)) {
			key = KEY_UNDO;
			break;
		}
		
		else if (_keytest(RR_LEFT)) {
			key = KEY_LEFT;
			break;
		}
		else if (_keytest(RR_RIGHT)) {
			key = KEY_RIGHT;
			break;
		}
		else if (_keytest(RR_UP)) {
			key = KEY_UP;
			break;
		}
		else if (_keytest(RR_DOWN)) {
			key = KEY_DOWN;
			break;
		}
	
		// Right (or Left) + [2nd]
		else if (_keytest(RR_2ND)){
			while (_rowread(0)) {
				if (_keytest(RR_LEFT)) {
					while (_keytest(RR_LEFT)) continue;
					return KEY_LEFT2ND;
				}
				else if (_keytest(RR_RIGHT)) {
					while (_keytest(RR_RIGHT)) continue;
					return KEY_RIGHT2ND;
				}
			}
			break;
		} 
	
		else if (_keytest(RR_PLUS)) {
			while (_rowread(0)) continue;
			OSContrastUp();
		}
		else if (_keytest(RR_MINUS)) {
			while (_rowread(0)) continue;
			OSContrastDn();
		}
		
	} while (TRUE);
	
	// Clear the keyboard queue
	while (_rowread(0)) continue;
	
	return key;		
}